package java.nio.file;

import java.util.NoSuchElementException;
import java.io.UncheckedIOException;
import java.io.IOException;
import java.util.Arrays;
import java.io.Closeable;
import java.util.Iterator;

class FileTreeIterator implements Iterator<FileTreeWalker.Event>, Closeable
{
    private final FileTreeWalker walker;
    private FileTreeWalker.Event next;
    
    FileTreeIterator(Path path, int n, FileVisitOption... a) throws IOException {
        this.walker = new FileTreeWalker(Arrays.asList(a), n);
        this.next = this.walker.walk(path);
        assert this.next.type() == FileTreeWalker.EventType.START_DIRECTORY;
        IOException ioeException = this.next.ioeException();
        if (ioeException != null) {
            throw ioeException;
        }
        return;
    }
    
    private void fetchNextIfNeeded() {
        if (this.next == null) {
            FileTreeWalker.Event next = this.walker.next();
            while (next != null) {
                IOException ioeException = next.ioeException();
                if (ioeException != null) {
                    throw new UncheckedIOException(ioeException);
                }
                if (next.type() != FileTreeWalker.EventType.END_DIRECTORY) {
                    this.next = next;
                    return;
                }
                next = this.walker.next();
            }
        }
    }
    
    @Override
    public boolean hasNext() {
        if (!this.walker.isOpen()) {
            throw new IllegalStateException();
        }
        this.fetchNextIfNeeded();
        return this.next != null;
    }
    
    @Override
    public FileTreeWalker.Event next() {
        if (!this.walker.isOpen()) {
            throw new IllegalStateException();
        }
        this.fetchNextIfNeeded();
        if (this.next == null) {
            throw new NoSuchElementException();
        }
        FileTreeWalker.Event next = this.next;
        this.next = null;
        return next;
    }
    
    @Override
    public void close() {
        this.walker.close();
    }
}
